package sun.security.krb5;

import sun.security.krb5.internal.KRBError;
import sun.security.krb5.internal.Krb5;
import java.util.Arrays;
import sun.security.jgss.krb5.Krb5Util;
import java.io.IOException;
import sun.security.krb5.internal.crypto.EType;
import sun.security.krb5.internal.PAData;
import javax.security.auth.kerberos.KeyTab;
import sun.security.krb5.internal.HostAddresses;
import sun.security.krb5.internal.KerberosTime;
import sun.security.krb5.internal.KDCOptions;

public final class KrbAsReqBuilder
{
    private KDCOptions options;
    private PrincipalName cname;
    private PrincipalName refCname;
    private PrincipalName sname;
    private KerberosTime from;
    private KerberosTime till;
    private KerberosTime rtime;
    private HostAddresses addresses;
    private final char[] password;
    private final KeyTab ktab;
    private PAData[] paList;
    private KrbAsReq req;
    private KrbAsRep rep;
    private KrbAsReqBuilder.State state;
    
    private void init(PrincipalName principalName) throws KrbException {
        this.cname = principalName;
        this.refCname = principalName;
        this.state = KrbAsReqBuilder.State.INIT;
    }
    
    public KrbAsReqBuilder(PrincipalName principalName, KeyTab ktab) throws KrbException {
        this.init(principalName);
        this.ktab = ktab;
        this.password = null;
    }
    
    public KrbAsReqBuilder(PrincipalName principalName, char[] array) throws KrbException {
        this.init(principalName);
        this.password = (char[])array.clone();
        this.ktab = null;
    }
    
    public EncryptionKey[] getKeys(boolean b) throws KrbException {
        this.checkState(b ? KrbAsReqBuilder.State.REQ_OK : KrbAsReqBuilder.State.INIT, "Cannot get keys");
        if (this.password != null) {
            int[] defaults = EType.getDefaults("default_tkt_enctypes");
            EncryptionKey[] array = new EncryptionKey[defaults.length];
            String s = null;
            try {
                for (int i = 0; i < defaults.length; ++i) {
                    PAData.SaltAndParams saltAndParams = PAData.getSaltAndParams(defaults[i], this.paList);
                    if (saltAndParams != null) {
                        if (defaults[i] != 23 && saltAndParams.salt != null) {
                            s = saltAndParams.salt;
                        }
                        array[i] = EncryptionKey.acquireSecretKey(this.cname, this.password, defaults[i], saltAndParams);
                    }
                }
                if (s == null) {
                    s = this.cname.getSalt();
                }
                for (int j = 0; j < defaults.length; ++j) {
                    if (array[j] == null) {
                        array[j] = EncryptionKey.acquireSecretKey(this.password, s, defaults[j], (byte[])null);
                    }
                }
            }
            catch (IOException cause) {
                KrbException ex = new KrbException(909);
                ex.initCause(cause);
                throw ex;
            }
            return array;
        }
        throw new IllegalStateException("Required password not provided");
    }
    
    public void setOptions(KDCOptions options) {
        this.checkState(KrbAsReqBuilder.State.INIT, "Cannot specify options");
        this.options = options;
    }
    
    public void setTarget(PrincipalName sname) {
        this.checkState(KrbAsReqBuilder.State.INIT, "Cannot specify target");
        this.sname = sname;
    }
    
    public void setAddresses(HostAddresses addresses) {
        this.checkState(KrbAsReqBuilder.State.INIT, "Cannot specify addresses");
        this.addresses = addresses;
    }
    
    private KrbAsReq build(EncryptionKey encryptionKey, KrbAsReqBuilder.ReferralsState referralsState) throws KrbException, IOException {
        PAData[] array = null;
        int[] array2;
        if (this.password != null) {
            array2 = EType.getDefaults("default_tkt_enctypes");
        }
        else {
            EncryptionKey[] keysFromJavaxKeyTab = Krb5Util.keysFromJavaxKeyTab(this.ktab, this.cname);
            array2 = EType.getDefaults("default_tkt_enctypes", keysFromJavaxKeyTab);
            for (int length = keysFromJavaxKeyTab.length, i = 0; i < length; ++i) {
                keysFromJavaxKeyTab[i].destroy();
            }
        }
        this.options = ((this.options == null) ? new KDCOptions() : this.options);
        if (referralsState.isEnabled()) {
            if (referralsState.sendCanonicalize()) {
                this.options.set(15, true);
            }
            array = new PAData[] { new PAData(149, new byte[0]) };
        }
        else {
            this.options.set(15, false);
        }
        return new KrbAsReq(encryptionKey, this.options, this.refCname, this.sname, this.from, this.till, this.rtime, array2, this.addresses, array);
    }
    
    private KrbAsReqBuilder resolve() throws KrbException, Asn1Exception, IOException {
        if (this.ktab != null) {
            this.rep.decryptUsingKeyTab(this.ktab, this.req, this.cname);
        }
        else {
            this.rep.decryptUsingPassword(this.password, this.req, this.cname);
        }
        if (this.rep.getPA() != null) {
            if (this.paList == null || this.paList.length == 0) {
                this.paList = this.rep.getPA();
            }
            else {
                int length = this.rep.getPA().length;
                if (length > 0) {
                    int length2 = this.paList.length;
                    this.paList = (PAData[])Arrays.copyOf(this.paList, this.paList.length + length);
                    System.arraycopy(this.rep.getPA(), 0, this.paList, length2, length);
                }
            }
        }
        return this;
    }
    
    private KrbAsReqBuilder send() throws KrbException, IOException {
        int n = 0;
        Object o = null;
        EncryptionKey encryptionKey = null;
        KrbAsReqBuilder.ReferralsState referralsState = new KrbAsReqBuilder.ReferralsState(this);
        while (true) {
            if (referralsState.refreshComm()) {
                o = new KdcComm(this.refCname.getRealmAsString());
            }
            try {
                this.req = this.build(encryptionKey, referralsState);
                this.rep = new KrbAsRep(o.send(this.req.encoding()));
                return this;
            }
            catch (KrbException ex) {
                if (n == 0 && (ex.returnCode() == 24 || ex.returnCode() == 25)) {
                    if (Krb5.DEBUG) {
                        System.out.println("KrbAsReqBuilder: PREAUTH FAILED/REQ, re-send AS-REQ");
                    }
                    n = 1;
                    KRBError error = ex.getError();
                    int preferredEType = PAData.getPreferredEType(error.getPA(), EType.getDefaults("default_tkt_enctypes")[0]);
                    if (this.password == null) {
                        EncryptionKey[] keysFromJavaxKeyTab = Krb5Util.keysFromJavaxKeyTab(this.ktab, this.cname);
                        encryptionKey = EncryptionKey.findKey(preferredEType, keysFromJavaxKeyTab);
                        if (encryptionKey != null) {
                            encryptionKey = (EncryptionKey)encryptionKey.clone();
                        }
                        for (int length = keysFromJavaxKeyTab.length, i = 0; i < length; ++i) {
                            keysFromJavaxKeyTab[i].destroy();
                        }
                    }
                    else {
                        encryptionKey = EncryptionKey.acquireSecretKey(this.cname, this.password, preferredEType, PAData.getSaltAndParams(preferredEType, error.getPA()));
                    }
                    this.paList = error.getPA();
                }
                else if (referralsState.handleError(ex)) {
                    encryptionKey = null;
                    n = 0;
                }
                else {
                    throw ex;
                }
            }
        }
    }
    
    public KrbAsReqBuilder action() throws KrbException, Asn1Exception, IOException {
        this.checkState(KrbAsReqBuilder.State.INIT, "Cannot call action");
        this.state = KrbAsReqBuilder.State.REQ_OK;
        return this.send().resolve();
    }
    
    public Credentials getCreds() {
        this.checkState(KrbAsReqBuilder.State.REQ_OK, "Cannot retrieve creds");
        return this.rep.getCreds();
    }
    
    public sun.security.krb5.internal.ccache.Credentials getCCreds() {
        this.checkState(KrbAsReqBuilder.State.REQ_OK, "Cannot retrieve CCreds");
        return this.rep.getCCreds();
    }
    
    public void destroy() {
        this.state = KrbAsReqBuilder.State.DESTROYED;
        if (this.password != null) {
            Arrays.fill(this.password, '\0');
        }
    }
    
    private void checkState(KrbAsReqBuilder.State obj, String str) {
        if (this.state != obj) {
            throw new IllegalStateException(str + " at " + obj + " state");
        }
        return;
    }
    
    private enum State
    {
        INIT, 
        REQ_OK, 
        DESTROYED;
    }
    
    static final class ReferralsState
    {
        private static boolean canonicalizeConfig;
        private boolean enabled;
        private boolean sendCanonicalize;
        private boolean isEnterpriseCname;
        private int count;
        private boolean refreshComm;
        private KrbAsReqBuilder reqBuilder;
        
        static void initStatic() {
            KrbAsReqBuilder.ReferralsState.canonicalizeConfig = false;
            try {
                KrbAsReqBuilder.ReferralsState.canonicalizeConfig = (Config.getInstance().getBooleanObject(new String[] { "libdefaults", "canonicalize" }) == Boolean.TRUE);
            }
            catch (KrbException ex) {
                if (Krb5.DEBUG) {
                    System.out.println("Exception in getting canonicalize, using default value " + Boolean.valueOf(KrbAsReqBuilder.ReferralsState.canonicalizeConfig) + ": " + ex.getMessage());
                }
            }
        }
        
        ReferralsState(KrbAsReqBuilder reqBuilder) throws KrbException {
            this.reqBuilder = reqBuilder;
            this.sendCanonicalize = KrbAsReqBuilder.ReferralsState.canonicalizeConfig;
            this.isEnterpriseCname = (reqBuilder.refCname.getNameType() == 10);
            this.updateStatus();
            if (!this.enabled && this.isEnterpriseCname) {
                throw new KrbException("NT-ENTERPRISE principals only allowed when referrals are enabled.");
            }
            this.refreshComm = true;
            return;
        }
        
        private void updateStatus() {
            this.enabled = (!Config.DISABLE_REFERRALS && (this.isEnterpriseCname || this.sendCanonicalize));
        }
        
        boolean handleError(KrbException ex) throws RealmException {
            if (this.enabled) {
                if (ex.returnCode() == 68) {
                    Realm clientRealm = ex.getError().getClientRealm();
                    if (clientRealm != null && !clientRealm.toString().isEmpty() && this.count < Config.MAX_REFERRALS) {
                        this.reqBuilder.refCname = new PrincipalName(this.reqBuilder.refCname.getNameType(), this.reqBuilder.refCname.getNameStrings(), clientRealm);
                        this.refreshComm = true;
                        ++this.count;
                        return true;
                    }
                }
                if (this.count < Config.MAX_REFERRALS && this.sendCanonicalize) {
                    if (Krb5.DEBUG) {
                        System.out.println("KrbAsReqBuilder: AS-REQ failed. Retrying with CANONICALIZE false.");
                    }
                    this.sendCanonicalize = false;
                    this.updateStatus();
                    return true;
                }
            }
            return false;
        }
        
        boolean refreshComm() {
            int refreshComm = this.refreshComm;
            this.refreshComm = false;
            return refreshComm;
        }
        
        boolean isEnabled() {
            return this.enabled;
        }
        
        boolean sendCanonicalize() {
            return this.sendCanonicalize;
        }
        
        static {
            initStatic();
        }
    }
}
